@@ -93,7 +93,12 @@ def validate_cuda_device(location):
 def _cuda_deserialize(obj, location):
     if location.startswith('cuda'):
         device = validate_cuda_device(location)
-        return obj.cuda(device)
+        if getattr(obj, "_torch_load_uninitialized", False):
+            storage_type = getattr(torch.cuda, type(obj).__name__)
+            with torch.cuda.device(device):
+                return storage_type(obj.size())
+        else:
+            return obj.cuda(device)
 
 
 register_package(10, _cpu_tag, _cpu_deserialize)
@@ -298,7 +303,7 @@ def _save(obj, f, pickle_module, pickle_protocol):
         serialized_storages[key]._write_file(f, _should_read_directly(f))
 
 
-def load(f, map_location=None, pickle_module=pickle):
+def load(f, map_location=None, pickle_module=pickle, **pickle_load_args):
     """Loads an object saved with :func:`torch.save` from a file.
 
     :meth:`torch.load` uses Python's unpickling facilities but treats storages,
@@ -337,12 +342,24 @@ def load(f, map_location=None, pickle_module=pickle):
             locations
         pickle_module: module used for unpickling metadata and objects (has to
             match the pickle_module used to serialize file)
+        pickle_load_args: optional keyword arguments passed over to
+            ``pickle_module.load`` and ``pickle_module.Unpickler``, e.g.,
+            ``encoding=...``.
 
     .. note::
         When you call :meth:`torch.load()` on a file which contains GPU tensors, those tensors
         will be loaded to GPU by default. You can call `torch.load(.., map_location='cpu')`
         and then :meth:`load_state_dict` to avoid GPU RAM surge when loading a model checkpoint.
 
+    .. note::
+        In Python 3, when loading files saved by Python 2, you may encounter
+        ``UnicodeDecodeError: 'ascii' codec can't decode byte 0x...``. This is
+        caused by the difference of handling in byte strings in Python2 and
+        Python 3. You may use extra ``encoding`` keyword argument to specify how
+        these objects should be loaded, e.g., ``encoding='latin1'`` decodes them
+        to strings using ``latin1`` encoding, and ``encoding='bytes'`` keeps them
+        as byte arrays which can be decoded later with ``byte_array.decode(...)``.
+
     Example:
         >>> torch.load('tensors.pt')
         # Load all tensors onto the CPU
@@ -354,24 +371,26 @@ def load(f, map_location=None, pickle_module=pickle):
         # Map tensors from GPU 1 to GPU 0
         >>> torch.load('tensors.pt', map_location={'cuda:1':'cuda:0'})
         # Load tensor from io.BytesIO object
-        >>> with open('tensor.pt') as f:
+        >>> with open('tensor.pt', 'rb') as f:
                 buffer = io.BytesIO(f.read())
         >>> torch.load(buffer)
     """
     new_fd = False
     if isinstance(f, str) or \
-            (sys.version_info[0] == 2 and isinstance(f, unicode)) or \
-            (sys.version_info[0] == 3 and isinstance(f, pathlib.Path)):
+            (sys.version_info[0] == 2 and isinstance(f, unicode)):
         new_fd = True
         f = open(f, 'rb')
+    elif (sys.version_info[0] == 3 and isinstance(f, pathlib.Path)):
+        new_fd = True
+        f = f.open('rb')
     try:
-        return _load(f, map_location, pickle_module)
+        return _load(f, map_location, pickle_module, **pickle_load_args)
     finally:
         if new_fd:
             f.close()
 
 
-def _load(f, map_location, pickle_module):
+def _load(f, map_location, pickle_module, **pickle_load_args):
     deserialized_objects = {}
 
     if map_location is None:
@@ -450,24 +469,24 @@ def _load(f, map_location, pickle_module):
 
             tar.extract('storages', path=tmpdir)
             with open(os.path.join(tmpdir, 'storages'), 'rb', 0) as f:
-                num_storages = pickle_module.load(f)
+                num_storages = pickle_module.load(f, **pickle_load_args)
                 for i in range(num_storages):
-                    args = pickle_module.load(f)
+                    args = pickle_module.load(f, **pickle_load_args)
                     key, location, storage_type = args
                     obj = storage_type._new_with_file(f)
                     obj = restore_location(obj, location)
                     deserialized_objects[key] = obj
 
-                storage_views = pickle_module.load(f)
+                storage_views = pickle_module.load(f, **pickle_load_args)
                 for target_cdata, root_cdata, offset, size in storage_views:
                     root = deserialized_objects[root_cdata]
                     deserialized_objects[target_cdata] = root[offset:offset + size]
 
             tar.extract('tensors', path=tmpdir)
             with open(os.path.join(tmpdir, 'tensors'), 'rb', 0) as f:
-                num_tensors = pickle_module.load(f)
+                num_tensors = pickle_module.load(f, **pickle_load_args)
                 for _ in range(num_tensors):
-                    args = pickle_module.load(f)
+                    args = pickle_module.load(f, **pickle_load_args)
                     key, storage_id, original_tensor_type = args
                     storage = deserialized_objects[storage_id]
                     tensor_type = storage_to_tensor_type(storage)
@@ -481,16 +500,27 @@ def _load(f, map_location, pickle_module):
                     deserialized_objects[key] = tensor
 
             pickle_file = tar.extractfile('pickle')
-            unpickler = pickle_module.Unpickler(pickle_file)
+            unpickler = pickle_module.Unpickler(pickle_file, **pickle_load_args)
             unpickler.persistent_load = persistent_load
             result = unpickler.load()
             return result
 
     deserialized_objects = {}
 
+    def maybe_decode_ascii(bytes_str):
+        # When using encoding='bytes' in Py3, some **internal** keys stored as
+        # strings in Py2 are loaded as bytes. This function decodes them with
+        # ascii encoding, one that Py3 uses by default.
+        #
+        # NOTE: This should only be used on internal keys (e.g., `typename` and
+        #       `location` in `persistent_load` below!
+        if isinstance(bytes_str, bytes):
+            return bytes_str.decode('ascii')
+        return bytes_str
+
     def persistent_load(saved_id):
         assert isinstance(saved_id, tuple)
-        typename = saved_id[0]
+        typename = maybe_decode_ascii(saved_id[0])
         data = saved_id[1:]
 
         if typename == 'module':
@@ -500,9 +530,11 @@ def _load(f, map_location, pickle_module):
             return data[0]
         elif typename == 'storage':
             data_type, root_key, location, size, view_metadata = data
+            location = maybe_decode_ascii(location)
             if root_key not in deserialized_objects:
-                deserialized_objects[root_key] = restore_location(
-                    data_type(size), location)
+                obj = data_type(size)
+                obj._torch_load_uninitialized = True
+                deserialized_objects[root_key] = restore_location(obj, location)
             storage = deserialized_objects[root_key]
             if view_metadata is not None:
                 view_key, offset, view_size = view_metadata
@@ -529,19 +561,19 @@ def _load(f, map_location, pickle_module):
             # if not a tarfile, reset file offset and proceed
             f.seek(0)
 
-    magic_number = pickle_module.load(f)
+    magic_number = pickle_module.load(f, **pickle_load_args)
     if magic_number != MAGIC_NUMBER:
         raise RuntimeError("Invalid magic number; corrupt file?")
-    protocol_version = pickle_module.load(f)
+    protocol_version = pickle_module.load(f, **pickle_load_args)
     if protocol_version != PROTOCOL_VERSION:
         raise RuntimeError("Invalid protocol version: %s" % protocol_version)
 
-    _sys_info = pickle_module.load(f)
-    unpickler = pickle_module.Unpickler(f)
+    _sys_info = pickle_module.load(f, **pickle_load_args)
+    unpickler = pickle_module.Unpickler(f, **pickle_load_args)
     unpickler.persistent_load = persistent_load
     result = unpickler.load()
 
-    deserialized_storage_keys = pickle_module.load(f)
+    deserialized_storage_keys = pickle_module.load(f, **pickle_load_args)
 
     offset = f.tell() if f_should_read_directly else None
     for key in deserialized_storage_keys:
